@@ -1,13 +1,14 @@
 import os
 import threading
 from collections import defaultdict
-from typing import Any, Dict
+from typing import Any, ClassVar, Optional
+from urllib.parse import parse_qs, urlencode, urlsplit, urlunsplit
 
+from funcy import first, wrap_prop
+
+from dvc.utils.objects import cached_property
 from dvc_objects.fs.base import ObjectFileSystem
 from dvc_objects.fs.errors import ConfigError
-from funcy import cached_property, wrap_prop
-
-from .path import S3Path
 
 _AWS_CONFIG_PATH = os.path.join(os.path.expanduser("~"), ".aws", "config")
 
@@ -40,37 +41,69 @@ def human_readable_to_bytes(value: str) -> int:
 # pylint:disable=abstract-method
 class S3FileSystem(ObjectFileSystem):
     protocol = "s3"
-    REQUIRES = {"s3fs": "s3fs", "boto3": "boto3"}
+    REQUIRES: ClassVar[dict[str, str]] = {"s3fs": "s3fs", "boto3": "boto3"}
     PARAM_CHECKSUM = "etag"
 
-    _GRANTS = {
+    VERSION_ID_KEY = "versionId"
+
+    _GRANTS: ClassVar[dict[str, str]] = {
         "grant_full_control": "GrantFullControl",
         "grant_read": "GrantRead",
         "grant_read_acp": "GrantReadACP",
         "grant_write_acp": "GrantWriteACP",
     }
 
-    _TRANSFER_CONFIG_ALIASES = {
+    _TRANSFER_CONFIG_ALIASES: ClassVar[dict[str, str]] = {
         "max_queue_size": "max_io_queue",
         "max_concurrent_requests": "max_concurrency",
         "multipart_threshold": "multipart_threshold",
         "multipart_chunksize": "multipart_chunksize",
     }
 
-    @cached_property
-    def path(self) -> S3Path:
-        def _getcwd():
-            return self.fs.root_marker
-
-        return S3Path(self.sep, getcwd=_getcwd)
+    def getcwd(self):
+        return self.fs.root_marker
 
     @classmethod
-    def _get_kwargs_from_urls(cls, urlpath: str) -> Dict[str, Any]:
+    def split_version(cls, path: str) -> tuple[str, Optional[str]]:
+        parts = list(urlsplit(path))
+        query = parse_qs(parts[3])
+        if cls.VERSION_ID_KEY in query:
+            version_id = first(query[cls.VERSION_ID_KEY])
+            del query[cls.VERSION_ID_KEY]
+            parts[3] = urlencode(query)
+        else:
+            version_id = None
+        return urlunsplit(parts), version_id
+
+    @classmethod
+    def join_version(cls, path: str, version_id: Optional[str]) -> str:
+        parts = list(urlsplit(path))
+        query = parse_qs(parts[3])
+        if cls.VERSION_ID_KEY in query:
+            raise ValueError("path already includes a version query")
+        parts[3] = f"{cls.VERSION_ID_KEY}={version_id}" if version_id else ""
+        return urlunsplit(parts)
+
+    @classmethod
+    def version_path(cls, path: str, version_id: Optional[str]) -> str:
+        path, _ = cls.split_version(path)
+        return cls.join_version(path, version_id)
+
+    @classmethod
+    def coalesce_version(
+        cls, path: str, version_id: Optional[str]
+    ) -> tuple[str, Optional[str]]:
+        path, path_version_id = cls.split_version(path)
+        versions = {ver for ver in (version_id, path_version_id) if ver}
+        if len(versions) > 1:
+            raise ValueError("Path version mismatch: '{path}', '{version_id}'")
+        return path, (versions.pop() if versions else None)
+
+    @classmethod
+    def _get_kwargs_from_urls(cls, urlpath: str) -> dict[str, Any]:
         ret = super()._get_kwargs_from_urls(urlpath)
         url_query = ret.get("url_query")
         if url_query is not None:
-            from urllib.parse import parse_qs
-
             parsed = parse_qs(url_query)
             if "versionId" in parsed:
                 ret["version_aware"] = True
@@ -178,9 +211,7 @@ class S3FileSystem(ObjectFileSystem):
 
         # config kwargs
         session_config = login_info["config_kwargs"]
-        session_config["s3"] = self._load_aws_config_file(
-            login_info["profile"]
-        )
+        session_config["s3"] = self._load_aws_config_file(login_info["profile"])
 
         shared_creds = config.get("credentialpath")
         if shared_creds:
@@ -209,7 +240,10 @@ class S3FileSystem(ObjectFileSystem):
     def fs(self):
         from s3fs import S3FileSystem as _S3FileSystem
 
-        return _S3FileSystem(**self.fs_args)
+        s3_filesystem = _S3FileSystem(**self.fs_args)
+        s3_filesystem.connect()
+
+        return s3_filesystem
 
     @classmethod
     def _strip_protocol(cls, path: str) -> str:
